;*****************************************************************************
;* LEONID.ASM -- Alef Null DSP CARD 4 monitor interface                      *
;*							                     *
;* Here are equates for services provided by DSP CARD 4                      *
;* monitor.						                     *
;*							                     *
;* Copyright (C) 1992-1995 by Alef Null. All rights reserved.		     *
;* Author(s): Jarkko Vuori, OH2LNS			                     *
;* Modification(s):					                     *
;  EVM addresses  -- JBF				                     *
;  MICGAIN	  -- JBF 8/17/95			                     *
;  Disable/Enable interrupts in opencd -- JBF 2/27/96	                     *
;  Define different memory and clock crystals				     *
;*****************************************************************************
;*********************    I M P O R T A N T    *******************************
;*****************************************************************************
; As a user you must:
; 1).....Choose the card that you are using.,
; 2).....Select the clock frequency that you wish to use.
; 3).....Select the SCI baudrate you wish to use
;*****************************************************************************
;*********************   DO IT HERE AND ONLY HERE   **************************
;*****************************************************************************

EVM56K  equ 1
                                ;1 => EVM56002 - standard
				;2 => EVM56002 - expanded

;-----------------------------------------------------------------------------
; Select EVM clock speed
; Some 66MHz parts will run up to 80 MHz
;-----------------------------------------------------------------------------
;xtal   equ     32000000         ; 32 MHz for most accurate SCI baudrates
xtal   equ     40000000         ; 40 MHz for standard EVM
;xtal   equ     66000000         ; 66 MHz for new EVM's !!!NOT DIVISIBLE BY 4!!!
;xtal    equ     68000000        ; 68 MHz experimental
;xtal   equ     80000000         ; 80 MHz experimental

;----------------------------------------------------------------------------
; SCI parameters
; (Choose your default SCI baudrate)
;----------------------------------------------------------------------------
baud   equ     19200                               ; SCI baud rate
;*****************************************************************************
hbeat   equ     72.0                                ; heartbeat blinking rate (pulses/min)

;-----------------------------------------------------------------------------
; EVM specific parameters
;-----------------------------------------------------------------------------
        if (EVM56K==2)          ; Expanded EVM
;-----------------------------------------------------------------------------
; On the expanded EVM56K, all memory is independant
;-----------------------------------------------------------------------------
ramx    equ     $0100           ; external X RAM st address
ramxlen equ     $4000-$0100     ; external X RAM length
ramy    equ     $0100           ; external Y RAM st address
ramylen equ     $4000-$0100     ; external Y RAM length
ramp    equ     $0100           ; external P RAM st address
ramplen equ     $8000-$0100     ; external P RAM length

	else                    ; Standard EVM

;-----------------------------------------------------------------------------
; On the standard EVM56K, P and X shares the same space, so we arbitrarily
; let P use 0100 - 1FFF, X use 2000 - 3FFF, Y is separate 0 - 3FFF
;-----------------------------------------------------------------------------
ramx    equ     $2000           ; external X RAM st address
ramxlen equ     $4000-$0100     ; external X RAM length
ramy    equ     $0100           ; external Y RAM st address
ramylen equ     $4000-$0100     ; external Y RAM length
ramp    equ     $0200           ; external P RAM st address
ramplen equ     $2000-$0200     ; external P RAM length
	endif



;-----------------------------------------------------------------------------
; Start of the user program and data areas
;-----------------------------------------------------------------------------
user_code equ	$0040  	    	; user P-memory starting address
user_data equ	$0018		; use X- and Y-memories starting address

	if (EVM56K==2)		; EVMs allocate buffers differently
topmem	equ    $6800 		; upper memory used for booting
kiss_pars equ	topmem 		; and SCI buffers
mybuflen equ	2048	  	; buffer size
	endif
	if (EVM56K==1)		; standard EVM
topmem equ	$1400
kiss_pars equ	topmem
mybuflen	equ	1024
	endif

opensc	macro
	jsr	opensci
	endm

;----------------------------------------------------------------------------
; Put a byte in x0 to the serial output
;-----------------------------------------------------------------------------
putc    macro
        jsr     putchar
        endm
;-----------------------------------------------------------------------------
; Request a character from the serial input
; waits until character given and returns it in x0
;-----------------------------------------------------------------------------
getc	macro
_gc1	wait
        jsr     getchar
	jcs	_gc1
	endm

;-----------------------------------------------------------------------------
; Look if there are characters waiting at the serial input
; If specified, wait for a given time (in seconds)
; returns C if there are no data available
;	 NC if there are data, and received data is placed to x0
;-----------------------------------------------------------------------------
lookc	macro	time
	if time==0
            jsr     getchar
	else
	    move    #>@cvi(time*19200),x0   ; set timer
            jsr     stimer
_cwait	    wait
            jsr     getchar                 ; check if we have a character available
	    jcc     _cfnd
	    jset    #4,y:$0002,_cwait	    ; no, are we tired for waiting
	    ori     #$01,ccr		    ; yes, ensure that C condition is met
_cfnd	    nop
	endif
	endm


;-----------------------------------------------------------------------------
; Wait for the given time (in seconds)
;-----------------------------------------------------------------------------
sleep	macro	time
	move		    #@cvi(time*19200.0),x0
        jsr     stimer
_slp1	wait
	jset	#4,y:$0002,_slp1
	endm

;-----------------------------------------------------------------------------

; macro for green LED handling
copled  macro   mode
	b\mode  #14,x:m_pbd
	endm

; macro for red LED handling
cmdled  macro   mode
	b\mode  #13,x:m_pbd
	endm

; macro for immediate move
movi    macro   data,dest
	move                data,a1
	move                a1,dest
	endm

; macro for entering interrupt service routine
; stores x0,x1 and a registers
enter   macro   contex
	move                x,l:<contex+0
	move                a10,l:<contex+1
	endm

; macro for leaving interrupt service routine
; restores x0,x1 and a registers
leave   macro   contex
	move                l:<contex+0,x
	move                l:<contex+1,a10
	rti
	endm

; CRC calculation routine
; data in the LSB of a
crc     macro   rem
	move                #>1,x0
	and     x0,a        rem,x0
	eor     x0,a
	lsr     a           #>poly,x0
	jcc     _crc1
	eor     x0,a
_crc1   move                a1,rem
	endm

; byte CRC calculation routine
; byte in x0, result in x:<crcrem
crcbyte macro
	move    x0,b1
	move                #>$000001,x1
	move                #>poly,y1
	do      #8,_crc2

	move    b1,a1                               ; first LSB to remainder
	and     x1,a        x:<crcrem,y0
	eor     y0,a

	lsr     a                                   ; XOR if needed
	jcc     _crc1
	eor     y1,a
_crc1   lsr     b           a1,x:<crcrem
_crc2
	endm


