; codec.asm
;-----------------------------------------------------------------------------
;               CODEC CS4215 handling for the DSP56002EVM
;
; V1.0 3.Jan.1997
;-----------------------------------------------------------------------------
;
	page	132

        section codec

        xdef    opencodec,closecodec

        include 'codec.inc'
        include 'ioequlc.inc'
;        org     p:

;****************************************************************************
;*     Open codec driver for EVM   *
;
;   r7 - address of the modulo buffer (x: A/D, y: D/A)
;   m7 - length of the modulo buffer
;   x0 - samping rate:
;
;****************************************************************************
;
;      portc usage:
;       bit8: SSI TX (from DSP to Codec)
;       bit7:
;       bit6:
;       bit5:
;       bit4: codec reset (from DSP to Codec)
;       bit3:
;       bit2: data/control bar
;             0=control
;             1=data
;
;
;  PROGRAM OUTLINE:
;
;1 program fsync and sclk == output
;2 write pc0 = 0 (control mode)
;3 send 64 bit frame x times, with dcb bit = 0, keep doing until read back as 0
;4 send 64 bit frame x times, with dcb bit = 1, keep doing until read back as 1
;5 re-program fsync and sclk == input
;6 write pc0 = 1 (data mode)
;7 receive/send data (echo slots 1,2,3,4; slots 5,6,7,8 == DIP switched)
;
;      initialize ssi -- fsync and sclk ==> outputs
;
;
; sample rate is passed in x0
; crystal selection is passed in x1
;
opencodec
	move    #CTRL_WD_12,a1          ; CODEC control words
	move    a1,y:<cryconf
	move    #CTRL_WD_34,a1
	move    a1,y:<cryconf+1
	move    #CTRL_WD_56,a1
	move    a1,y:<cryconf+2
	move    #CTRL_WD_78,a1
	move    a1,y:<cryconf+3

	movep   #$0003,x:m_pcc  ;  turn off ssi port (keep SCI on)
	movep   #$4303,x:m_cra  ;  40MHz/16 = 2.5MHz SCLK, WL=16 bits, 4W/F
	movep   #$3B30,x:m_crb  ; RIE,TIE,RE,TE, NTWK, SYN, FSR/RSR->bit

	bclr    #4,x:m_pcd      ; RESET~ ....
	bclr    #2,x:m_pcd      ; D/C~   ...... 0 ==> Control mode
				;----reset delay for codec ----
	do      #500,_delay_loop
	rep     #2000           ; 100 us delay
	nop
_delay_loop
	bset    #4,x:m_pcd      ; RESET~ = 1

	movep   #$01E3,x:m_pcc  ; Turn on ssi port (keep SCI on)

; send control blocks to the codec until we get valid responce from it
						      ; add sampling rate and set CLB = 0
	move                #>%1110101101000111<<8,x1 ; add MICGAIN, HPF,
						      ; sampling rate,
						      ; set CLB = 0
	move                y:<cryconf,a1
	and     x1,a
	or      x0,a        #cryconf,r0
	move                a1,y:<cryconf
	move                #4-1,m0

confcod jsr     outblk          ; send control info until CLB is low
	jset    #2+16,x:tmpblk+2,confcod

; codec is configured, send final control block
	bset    #2+16,y:<cryconf+0                  ; set CLB = 1
	do      #200,confok                         ; at least two frames after CLB high
	jsr     outblk                              ; and ensure that at least 50 ms elapsed after leaving from PDN state
	nop
confok  move                #-1,m0

	movep   #$0003,x:m_pcc  ;  turn off ssi port (keep SCI on)

;*****************************************************************************
;  now CLB should be 1 -- re-program fsync and sclk direction (i/p)
;
	movep   #$4303,x:m_cra  ; 16bits,4 word/frame, /2/4/2=2.5 MHz
	bset    #2,x:m_pcd      ; D/C~ = 1 ==> Data mode
	movep   #$01E3,x:m_pcc  ;  turn on ssi port (keep SCI on)

waitsyn jclr    #m_tde,x:m_sr,waitsyn               ; wait for the frame sync
	jset    #m_tfs,x:m_sr,frmsync
	movep               x:m_rx,x:m_tsr
	jmp     waitsyn

frmsync do      #4-1,flshfrm                        ; then get rid of the remaining data
_loop1  jclr    #m_tde,x:m_sr,_loop1
	movep               y:(r7)+,x:m_tx
	movep               x:m_rx,x:(r7)
flshfrm

	do      #192*4,waitcal                      ; wait for calibration
_loop2  jclr    #m_tde,x:m_sr,_loop2
	movep               y:(r7)+,x:m_tx
	movep               x:m_rx,x:(r7)
waitcal

	movep   #$7b00,x:m_crb      ; enable transmit interrupts
	rts

; Output one 256 bit block (codec in the first time slot)
outblk  do      #4,outblk1                          ; actual data transfer on the first slot only
_loop   jclr    #m_tde,x:m_sr,_loop
	movep               x:m_rx,x:(r0)
	movep               y:(r0)+,x:m_tx

outblk1 do      #16-4,outblk2                       ; idling remainig slots
_loop   jclr    #m_tde,x:m_sr,_loop
	movep               a1,x:m_tsr

outblk2 rts

;****************************
;*     Close codec driver   *
;****************************
; close SSI interface and set codec to power down mode
closecodec
	movep               #$0003,x:m_pcc          ; TXD,RXD
	movep               #$001c,x:m_pcddr        ; SCLK,SC0,SC1 as output
	rts

; Crystal CS4215 configuration data
; Stereo, 16-bit linear, XTAL1, 64 bit/frame, generate SCLK and FSYNC
        org     y:$0004
cryconf dc      %0000000000000100<<8                ; Control Time Slot 1 & 2
	dc      %1001001000000000<<8                ; Control Time Slot 3 & 4
	dc      %0000000000000000<<8                ; Control Time Slot 5 & 6
	dc      %0000000000000000<<8                ; Control Time Slot 7 & 8

        org     x:$0004
tmpblk  dsm     4

        endsec  ; codec
        end